unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, BufDataset, sqldb, sqlite3conn,
  FileUtil, LR_Class, LR_DBSet, LR_View, LR_Desgn, Forms,
  Controls, Graphics, Dialogs, StdCtrls, DbCtrls, ExtCtrls, DBGrids,
  LazFileUtils, db;

type

  DrinkType = (dtWater, dtWine, dtBeer);

const
  DrinkTypeName: array[Low(DrinkType)..High(DrinkType)] of String =
    ('Wasser', 'Wein', 'Bier');

  DrinkTypePrice: array[Low(DrinkType)..High(DrinkType)] of Currency =
    (0.9, 1.5, 2.5);

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    DataAkten: TDataSource;
    DataSourceTelefon: TDataSource;
    DataSourceAdressen: TDataSource;
    DBGridAkten: TDBGrid;
    DBGridAdressen: TDBGrid;
    DBGridTelefon: TDBGrid;
    DBNavigatorTelefon: TDBNavigator;
    DBNavigatorAkten: TDBNavigator;
    DBNavigatorAdressen: TDBNavigator;
    LRDataSetTelefon: TfrDBDataSet;
    LRDataSetAdressen: TfrDBDataSet;
    LRDataSetAkten: TfrDBDataSet;
    frDesigner1: TfrDesigner;
    frPreview1: TfrPreview;
    Panel1: TPanel;
    PanelAkten: TPanel;
    PanelAdressen: TPanel;
    PanelTelefon: TPanel;
    Report: TfrReport;
    Connection: TSQLite3Connection;
    QueryAkten: TSQLQuery;
    QueryAdressen: TSQLQuery;
    QueryTelefon: TSQLQuery;
    Transaction: TSQLTransaction;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure QueryAdressenAfterOpen(DataSet: TDataSet);
    procedure QueryAdressenBeforeClose(DataSet: TDataSet);
    procedure QueryAktenAfterOpen(DataSet: TDataSet);
    procedure QueryAktenAfterScroll(DataSet: TDataSet);
    procedure QueryAktenBeforeClose(DataSet: TDataSet);
    procedure QueryTelefonAfterOpen(DataSet: TDataSet);
    procedure QueryTelefonBeforeClose(DataSet: TDataSet);
    procedure ReportGetValue(const ParName: String; var ParValue: Variant);
  private
    procedure ConnectDatabase;
    procedure ConnectQueries;
    procedure CreateTables;
    procedure FillSomeData;
    function GetTelephoneNumbers: String;
    procedure LoadReport;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  ConnectDatabase;
  CreateTables;
  ConnectQueries;
  if QueryAkten.RecordCount < 1 then  // First run
    FillSomeData;
  LoadReport;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  Report.DesignReport;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Transaction.Commit;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  Report.ShowReport;
end;

procedure TForm1.QueryAdressenAfterOpen(DataSet: TDataSet);
var
  i: Integer;
begin
  for i := 0 to QueryAdressen.FieldCount - 1 do
    case QueryAdressen.Fields[i].FieldName of
      'adress':
        begin
          DBGridAdressen.Columns.Items[i].SizePriority := 1;
          DBGridAdressen.Columns.Items[i].Title.Caption := 'Adresse';
        end;
      else
        begin
          DBGridAdressen.Columns.Items[i].SizePriority := 0;
          DBGridAdressen.Columns.Items[i].Width := 0;
        end;
    end;
end;

procedure TForm1.QueryAdressenBeforeClose(DataSet: TDataSet);
begin
  QueryAdressen.ApplyUpdates;
end;

procedure TForm1.QueryAktenAfterOpen(DataSet: TDataSet);
var
  i: Integer;
begin
  for i := 0 to QueryAkten.FieldCount - 1 do
    case QueryAkten.Fields[i].FieldName of
      'name':
        begin
          DBGridAkten.Columns.Items[i].SizePriority := 1;
          DBGridAkten.Columns.Items[i].Title.Caption := 'Name';
        end;
      else
        begin
          DBGridAkten.Columns.Items[i].SizePriority := 0;
          DBGridAkten.Columns.Items[i].Width := 0;
        end;
    end;
end;

procedure TForm1.QueryAktenAfterScroll(DataSet: TDataSet);
begin
  Report.ShowReport;
end;

procedure TForm1.QueryAktenBeforeClose(DataSet: TDataSet);
begin
  QueryAkten.ApplyUpdates;
end;

procedure TForm1.QueryTelefonAfterOpen(DataSet: TDataSet);
var
  i: Integer;
begin
  for i := 0 to QueryTelefon.FieldCount - 1 do
    case QueryTelefon.Fields[i].FieldName of
      'number':
        begin
          DBGridTelefon.Columns.Items[i].SizePriority := 1;
          DBGridTelefon.Columns.Items[i].Title.Caption := 'TelefonNr';
        end;
      else
        begin
          DBGridTelefon.Columns.Items[i].SizePriority := 0;
          DBGridTelefon.Columns.Items[i].Width := 0;
        end;
    end;
end;

procedure TForm1.QueryTelefonBeforeClose(DataSet: TDataSet);
begin
  QueryTelefon.ApplyUpdates;
end;

procedure TForm1.ReportGetValue(const ParName: String; var ParValue: Variant);
begin
  case ParName of
    'TelephoneNumbers':
    begin
      ParValue := GetTelephoneNumbers;
    end;
  end;
end;

procedure TForm1.ConnectDatabase;
begin
  try
    Connection.Connected := True;
  except
    on e: Exception do
      ShowMessage('TForm1.ConnectDatabase: ' + e.Message);
  end;
end;

procedure TForm1.ConnectQueries;
begin
  try
    QueryAkten.SQL.Text :=
      'SELECT akt_id, name FROM akten ORDER BY name ASC;';
    QueryAkten.Open;

    QueryAdressen.SQL.Text :=
      'SELECT adr_id, akt_id, adress FROM adressen ' +
      'WHERE adressen.akt_id = :akt_id ' +
      'ORDER BY adress ASC;';
    QueryAdressen.Open;

    QueryTelefon.SQL.Text :=
      'SELECT tel_id, akt_id, number FROM telefon ' +
      'WHERE telefon.akt_id = :akt_id ' +
      'ORDER BY number ASC;';
    QueryTelefon.Open;
  except
    on e: Exception do
      ShowMessage('TForm1.ConnectQueries: ' + e.Message);
  end;
end;

procedure TForm1.CreateTables;
begin
  try
    Connection.ExecuteDirect(
      'CREATE TABLE IF NOT EXISTS akten (' +
      'akt_id INTEGER PRIMARY KEY AUTOINCREMENT, ' +
      'name VARCHAR);');

    Connection.ExecuteDirect(
      'CREATE TABLE IF NOT EXISTS adressen (' +
      'adr_id INTEGER PRIMARY KEY AUTOINCREMENT, ' +
      'akt_id INTEGER REFERENCES akten(id) ON DELETE CASCADE, ' +
      'adress VARCHAR);');

    Connection.ExecuteDirect(
      'CREATE TABLE IF NOT EXISTS telefon (' +
      'tel_id INTEGER PRIMARY KEY AUTOINCREMENT, ' +
      'akt_id INTEGER REFERENCES akten(id) ON DELETE CASCADE, ' +
      'number VARCHAR);');
  except
    on e: Exception do
      ShowMessage('TForm1.CreateTables: ' + e.Message);
  end;
end;

procedure TForm1.FillSomeData;

  procedure FillSomeTelefon;
  var
    i: Integer;
  begin
    for i := 0 to Random(10) do
    begin
      QueryTelefon.Append;
      QueryTelefon.FieldByName('number').AsString  := '0' + IntToStr(Random(1000000000));
      QueryTelefon.FieldByName('akt_id').AsInteger := QueryAkten.FieldByName('akt_id').AsInteger;
      QueryTelefon.Insert;
      QueryTelefon.ApplyUpdates;
    end;
  end;

  procedure FillSomeAdressen;
  var
    i: Integer;
  begin
    for i := 0 to Random(10) do
    begin
      QueryAdressen.Append;
      QueryAdressen.FieldByName('adress').AsString  := 'Adresse ' + IntToStr(i + 1);
      QueryAdressen.FieldByName('akt_id').AsInteger := QueryAkten.FieldByName('akt_id').AsInteger;
      QueryAdressen.Insert;
      QueryAdressen.ApplyUpdates;
    end;
  end;

  procedure FillSomeAkte;
  begin
    QueryAkten.Append;
    QueryAkten.FieldByName('name').AsString := 'Akte Nr ' + IntToStr(QueryAkten.RecordCount + 1);
    QueryAkten.Insert;
    QueryAkten.ApplyUpdates;
    FillSomeAdressen;
    FillSomeTelefon;
  end;

var
  i: Integer;
begin
  for i := 0 to 9 do
    FillSomeAkte;
end;

function TForm1.GetTelephoneNumbers: String;
begin
  Result := '';
  QueryTelefon.First;
  while not QueryTelefon.EOF do
  begin
    if Result = '' then
      Result := QueryTelefon.FieldByName('number').AsString
    else
      Result := Result + LineEnding + QueryTelefon.FieldByName('number').AsString;
    QueryTelefon.Next;
  end;
end;

procedure TForm1.LoadReport;
var
  AFileName: String;
begin
  try
    AFileName := GetCurrentDirUTF8 + PathDelim + 'test.lrf';
    if FileExists(AFileName) then
      Report.LoadFromFile(AFileName);
  except
    on e: Exception do
      ShowMessage('TForm1.LoadReport: ' + e.Message);
  end;
end;

end.

